@@ -12,9 +12,19 @@ import traceback
 import uuid
 from argparse import ArgumentDefaultsHelpFormatter
 
+import requests
+from stem.control import EventType
+
 import sbws.util.requests as requests_utils
 import sbws.util.stem as stem_utils
-from sbws.globals import HTTP_GET_HEADERS, SOCKET_TIMEOUT, fail_hard
+from sbws.globals import (
+    BWSCANNER_CC2,
+    HTTP_GET_HEADERS,
+    HTTP_POST_INITIAL_SIZE,
+    HTTP_POST_UL_KEY,
+    SOCKET_TIMEOUT,
+    fail_hard,
+)
 
 from .. import settings
 from ..lib.circuitbuilder import GapsCircuitBuilder as CB
@@ -89,7 +99,7 @@ def timed_recv_from_server(session, dest, byte_range):
     return True and the time it took to download. Otherwise return False and an
     exception."""
 
-    start_time = time.time()
+    start_time = time.monotonic()
     HTTP_GET_HEADERS["Range"] = byte_range
     # - response.elapsed "measures the time taken between sending the first
     #   byte of the request and finishing parsing the headers.
@@ -106,7 +116,7 @@ def timed_recv_from_server(session, dest, byte_range):
     except Exception as e:
         log.debug(e)
         return False, e
-    end_time = time.time()
+    end_time = time.monotonic()
     return True, end_time - start_time
 
 
@@ -118,7 +128,6 @@ def get_random_range_string(content_length, size):
     For example, for content_length of 100 and size 10, this function will
     return one of the following: '0-9', '1-10', '2-11', [...] '89-98', '90-99'
     """
-    assert size <= content_length
     # start can be anywhere in the content_length as long as it is **size**
     # bytes away from the end or more. Because range is [start, end) (doesn't
     # include the end value), add 1 to the end.
@@ -130,7 +139,6 @@ def get_random_range_string(content_length, size):
     # the largest index end should ever be should be less than the total length
     # of the content. For example, if content_length is 10, end could be
     # anywhere from 0 to 9.
-    assert end < content_length
     return "bytes={}-{}".format(start, end)
 
 
@@ -159,9 +167,7 @@ def measure_rtt_to_server(session, conf, dest, content_length):
                 data,
             )
             return None, data
-        assert success
         # data is an RTT
-        assert isinstance(data, float) or isinstance(data, int)
         rtts.append(data)
     return rtts, None
 
@@ -184,8 +190,6 @@ def measure_bandwidth_to_server(session, conf, dest, content_length):
         "max": conf.getfloat("scanner", "download_max"),
     }
     while len(results) < num_downloads and not settings.end_event.is_set():
-        assert expected_amount >= min_dl
-        assert expected_amount <= max_dl
         random_range = get_random_range_string(content_length, expected_amount)
         success, data = timed_recv_from_server(session, dest, random_range)
         if not success:
@@ -198,9 +202,7 @@ def measure_bandwidth_to_server(session, conf, dest, content_length):
                 data,
             )
             return None, data
-        assert success
         # data is a download time
-        assert isinstance(data, float) or isinstance(data, int)
         if _should_keep_result(
             expected_amount == max_dl, data, download_times
         ):
@@ -211,6 +213,97 @@ def measure_bandwidth_to_server(session, conf, dest, content_length):
     return results, None
 
 
+def upload_data(session, conf, dest, cont, circ_id):
+    """
+    Upload data to a destination URL over ``circ_id`` via HTTP POST request.
+
+    This function is equivalent to the HTTP HEAD and GET requests implemented
+    with the functions:
+
+    - ``sbws.lib.destination.connect_to_destination_over_circuit``
+      which in turn calls``sbws.lib.stem.attach_stream_to_circuit_listener``
+      and ``sbws.lib.stem.add_event_listener``: to listen for stream event.
+    - ``measure_bandwidth_to_server``: to calculate download sizes, not needed
+      anymore.
+    - ``timed_recv_from_server``: the measurement itself.
+
+    :return: measurement ``Result`` (or None) and error or None
+    :rtype: tuple(Result, str)
+
+    """
+    log.debug("Uploading data...")
+
+    size = conf.getint("scanner", "http_post_initial_size")
+    # The data to upload is so far zeros.
+    data = bytearray(size)
+
+    # Block other threads to attach an stream to the same circuit.
+    # Otherwise, if there're measurer threads trying to attach other streams,
+    # the controller will have several listener for the same event type
+    # (stream) and it might (will?) use the same listener (and circuit) for
+    # the new streams.
+    with stem_utils.stream_building_lock:
+        listener = stem_utils.attach_stream_to_circuit_listener(
+            cont, circ_id, BWSCANNER_CC2
+        )
+        stem_utils.add_event_listener(cont, listener, EventType.STREAM)
+        # There must be some stream (HTTP request) to actually attach the
+        # stream.
+        try:
+            session.head(dest.url, verify=dest.verify)
+        except requests.exceptions.RequestException as e:
+            dest.add_failure()
+            return None, "Could not connect to {} over circ {} {}: {}".format(
+                dest.url, circ_id, stem_utils.circuit_str(cont, circ_id), e
+            )
+        finally:
+            stem_utils.remove_event_listener(cont, listener)
+
+    try:
+        response = session.post(
+            dest.url,
+            files={conf.get("scanner", "payload_key"): data},
+            verify=dest.verify,
+        )
+    except requests.exceptions.RequestException as e:
+        dest.add_failure()
+        msg = "Could not connect to {} over circ {} {}: {}".format(
+            dest.url, circ_id, stem_utils.circuit_str(cont, circ_id), e
+        )
+        log.debug("%s: %s", msg, e)
+        return None, msg
+    except Exception as e:
+        dest.add_failure()
+        log.debug(e)
+        return None, e
+    finally:
+        log.debug("Finished uploading data.")
+
+    if response.status_code != requests.codes.ok:
+        dest.add_failure()
+        msg = (
+            "When sending HTTP POST to {}, we expected HTTP code {} "
+            "not {}".format(dest.url, requests.codes.ok, response.status_code)
+        )
+        log.debug(msg)
+        return None, msg
+
+    dest.add_success()
+    time_elapsed = response.elapsed.total_seconds()
+    results = [{"duration": time_elapsed, "amount": size}]
+    log.debug("Time uploading %s bytes: %s seconds.", size, time_elapsed)
+    log.debug(
+        "POST response elapsed seconds: %s.",
+        response.elapsed.total_seconds(),
+    )
+    measured_bandwidth = size / time_elapsed
+    log.debug(
+        "Measured bandwidth: %s Bytes/seconds",
+        measured_bandwidth,
+    )
+    return results, None
+
+
 def _pick_ideal_second_hop(relay, rl, relay_as_entry, candidates):
     """
     Sbws builds two hop circuits. Given the **relay** to measure, pick a helper
@@ -440,6 +533,17 @@ def select_helper_candidates(relay, rl, dest, relay_as_entry=True):
     return candidates
 
 
+def relay_update_xoff(relay, circ_id):
+    # Check `XOFF_RECV/SENT`
+    relay.update_xoff_recv(
+        settings.stream_event[circ_id].pop("XOFF_RECV", None)
+    )
+    relay.update_xoff_sent(
+        settings.stream_event[circ_id].pop("XOFF_SENT", None)
+    )
+    settings.stream_event.pop(circ_id)
+
+
 def measure_relay(args, conf, destinations, cb, rl, relay):
     """
     Select a Web server, a relay to build the circuit,
@@ -527,104 +631,119 @@ def measure_relay(args, conf, destinations, cb, rl, relay):
         relay.fingerprint,
         relay.nickname,
     )
-    # Make a connection to the destination
-    is_usable, usable_data = connect_to_destination_over_circuit(
-        dest, circ_id, s, cb.controller, dest._max_dl
-    )
 
-    # In the case that the relay was used as an exit, but could not exit
-    # to the Web server, try again using it as entry, to avoid that it would
-    # always fail when there's only one Web server.
-
-    if not is_usable and not relay_as_entry:
-        log.debug(
-            "Exit %s (%s) that can't exit all ips, with exit policy %s, failed"
-            " to connect to %s via circuit %s (%s). Reason: %s. Trying again "
-            "with it as entry.",
-            relay.fingerprint,
-            relay.nickname,
-            exit_policy,
-            dest.url,
-            circ_fps,
-            nicknames,
-            usable_data,
+    if rl.is_consensus_bwscanner_cc_2:
+        # 40130: Upload data instead of downloading it.
+        bw_results, reason = upload_data(s, conf, dest, cb.controller, circ_id)
+    else:
+        # Make a connection to the destination
+        is_usable, usable_data = connect_to_destination_over_circuit(
+            dest, circ_id, s, cb.controller, dest._max_dl
         )
-        relay_as_entry = True
-        # select new candidates as exit
-        candidates = select_helper_candidates(relay, rl, dest, relay_as_entry)
-        r = create_path_relay(relay, dest, rl, relay_as_entry, candidates)
-        if len(r) == 1:
-            return r
-        circ_fps, nicknames, exit_policy = r
-        circ_id, reason = cb.build_circuit(circ_fps)
-        if not circ_id:
-            log.info(
-                "Exit %s (%s) that can't exit all ips, failed to create "
-                " circuit as entry: %s (%s).",
+
+        # In the case that the relay was used as an exit, but could not exit
+        # to the Web server, try again using it as entry, to avoid that it
+        # would always fail when there's only one Web server.
+
+        if not is_usable and not relay_as_entry:
+            log.debug(
+                "Exit %s (%s) that can't exit all ips, with exit policy %s, "
+                "failed to connect to %s via circuit %s (%s). Reason: %s. "
+                "Trying again with it as entry.",
+                relay.fingerprint,
+                relay.nickname,
+                exit_policy,
+                dest.url,
+                circ_fps,
+                nicknames,
+                usable_data,
+            )
+            relay_as_entry = True
+            # select new candidates as exit
+            candidates = select_helper_candidates(
+                relay, rl, dest, relay_as_entry
+            )
+            r = create_path_relay(relay, dest, rl, relay_as_entry, candidates)
+            if len(r) == 1:
+                return r
+            circ_fps, nicknames, exit_policy = r
+            circ_id, reason = cb.build_circuit(circ_fps)
+            if not circ_id:
+                log.info(
+                    "Exit %s (%s) that can't exit all ips, failed to create "
+                    " circuit as entry: %s (%s).",
+                    relay.fingerprint,
+                    relay.nickname,
+                    circ_fps,
+                    nicknames,
+                )
+                return error_no_circuit(
+                    circ_fps, nicknames, reason, relay, dest, our_nick
+                )
+
+            log.debug(
+                "Built circuit with path %s (%s) to measure %s (%s)",
+                circ_fps,
+                nicknames,
+                relay.fingerprint,
+                relay.nickname,
+            )
+            is_usable, usable_data = connect_to_destination_over_circuit(
+                dest, circ_id, s, cb.controller, dest._max_dl
+            )
+        if not is_usable:
+            log.debug(
+                "Failed to connect to %s to measure %s (%s) via circuit "
+                "%s (%s). Exit policy: %s. Reason: %s.",
+                dest.url,
                 relay.fingerprint,
                 relay.nickname,
                 circ_fps,
                 nicknames,
+                exit_policy,
+                usable_data,
             )
-            return error_no_circuit(
-                circ_fps, nicknames, reason, relay, dest, our_nick
+            cb.close_circuit(circ_id)
+            return [
+                ResultErrorStream(
+                    relay, circ_fps, dest.url, our_nick, msg=usable_data
+                ),
+            ]
+        # FIRST: measure RTT
+        rtts, reason = measure_rtt_to_server(
+            s, conf, dest, usable_data["content_length"]
+        )
+        if rtts is None:
+            log.debug(
+                "Unable to measure RTT for %s (%s) to %s via circuit "
+                "%s (%s): %s",
+                relay.fingerprint,
+                relay.nickname,
+                dest.url,
+                circ_fps,
+                nicknames,
+                reason,
             )
+            cb.close_circuit(circ_id)
+            return [
+                ResultErrorStream(
+                    relay, circ_fps, dest.url, our_nick, msg=str(reason)
+                ),
+            ]
 
-        log.debug(
-            "Built circuit with path %s (%s) to measure %s (%s)",
-            circ_fps,
-            nicknames,
-            relay.fingerprint,
-            relay.nickname,
+        # SECOND: measure bandwidth
+        bw_results, reason = measure_bandwidth_to_server(
+            s, conf, dest, usable_data["content_length"]
         )
-        is_usable, usable_data = connect_to_destination_over_circuit(
-            dest, circ_id, s, cb.controller, dest._max_dl
-        )
-    if not is_usable:
-        log.debug(
-            "Failed to connect to %s to measure %s (%s) via circuit "
-            "%s (%s). Exit policy: %s. Reason: %s.",
-            dest.url,
-            relay.fingerprint,
-            relay.nickname,
-            circ_fps,
-            nicknames,
-            exit_policy,
-            usable_data,
-        )
-        cb.close_circuit(circ_id)
-        return [
-            ResultErrorStream(
-                relay, circ_fps, dest.url, our_nick, msg=usable_data
-            ),
-        ]
-    assert is_usable
-    assert "content_length" in usable_data
-    # FIRST: measure RTT
-    rtts, reason = measure_rtt_to_server(
-        s, conf, dest, usable_data["content_length"]
-    )
-    if rtts is None:
-        log.debug(
-            "Unable to measure RTT for %s (%s) to %s via circuit "
-            "%s (%s): %s",
-            relay.fingerprint,
-            relay.nickname,
-            dest.url,
-            circ_fps,
-            nicknames,
-            reason,
-        )
-        cb.close_circuit(circ_id)
-        return [
-            ResultErrorStream(
-                relay, circ_fps, dest.url, our_nick, msg=str(reason)
-            ),
-        ]
-    # SECOND: measure bandwidth
-    bw_results, reason = measure_bandwidth_to_server(
-        s, conf, dest, usable_data["content_length"]
-    )
+
+    # The `XOFF` events can only be received once an upload or download
+    # stream starts (in `measure_bandwidth_to_server` or
+    # `upload_data_multipart` functions). This is the reason why they're stored
+    # here and not before.
+    # The `XOFF` events are also independent on whether the stream end up
+    # failing or succeeding.
+    relay_update_xoff(relay, circ_id)
+
     if bw_results is None:
         log.debug(
             "Failed to measure %s (%s) via circuit %s (%s) to %s. Exit"
@@ -646,15 +765,16 @@ def measure_relay(args, conf, destinations, cb, rl, relay):
     cb.close_circuit(circ_id)
     # Finally: store result
     log.debug(
-        "Success measurement for %s (%s) via circuit %s (%s) to %s",
+        "Success measurement for %s (%s) via circuit %s (%s) to %s: %s",
         relay.fingerprint,
         relay.nickname,
         circ_fps,
         nicknames,
         dest.url,
+        bw_results,
     )
     return [
-        ResultSuccess(rtts, bw_results, relay, circ_fps, dest.url, our_nick),
+        ResultSuccess(None, bw_results, relay, circ_fps, dest.url, our_nick),
     ]
 
 
@@ -804,7 +924,7 @@ def main_loop(
     while not settings.end_event.is_set():
         log.debug("Starting a new measurement loop.")
         num_relays = 0
-        loop_tstart = time.time()
+        loop_tstart = time.monotonic()
 
         # Register relay fingerprints to the heartbeat module
         hbeat.register_consensus_fprs(relay_list.relays_fingerprints)
@@ -852,7 +972,7 @@ def main_loop(
         # Print the heartbeat message
         hbeat.print_heartbeat_message()
 
-        loop_tstop = time.time()
+        loop_tstop = time.monotonic()
         loop_tdelta = (loop_tstop - loop_tstart) / 60
         # At this point, we know the relays that were queued to be
         # measured.
@@ -1032,7 +1152,12 @@ def main(args, conf):
             min_dl,
         )
 
-    os.makedirs(conf.getpath("paths", "datadir"), exist_ok=True)
+    os.makedirs(conf.getpath("paths", "datadir"), mode=0o700, exist_ok=True)
+
+    # For now, no need to add these variables as config file options or args.
+    print("conf", conf)
+    conf["scanner"]["payload_key"] = str(HTTP_POST_UL_KEY)
+    conf["scanner"]["http_post_initial_size"] = str(HTTP_POST_INITIAL_SIZE)
 
     state = State(conf.getpath("paths", "state_fname"))
     state["scanner_started"] = now_isodt_str()
