@@ -75,8 +75,6 @@ def _get_user_config(args, conf=None):
             interpolation=ExtendedInterpolation(),
             converters={"path": _expand_path},
         )
-    else:
-        assert isinstance(conf, ConfigParser)
     if args.config:
         if not os.path.isfile(args.config):
             # XXX: The logger is not configured at this stage,
@@ -105,8 +103,6 @@ def _get_default_logging_config(conf=None):
             interpolation=ExtendedInterpolation(),
             converters={"path": _expand_path},
         )
-    else:
-        assert isinstance(conf, ConfigParser)
     return _extend_config(conf, DEFAULT_LOG_CONFIG_PATH)
 
 
@@ -140,7 +136,6 @@ def _can_log_to_file(conf):
 
 
 def configure_logging(args, conf):
-    assert isinstance(conf, ConfigParser)
     logger = "logger_sbws"
     # Set the correct handler(s) based on [logging] options
     handlers = set()
@@ -164,7 +159,7 @@ def configure_logging(args, conf):
         # the sbws command (like 'scanner' or 'generate') if possible, or to
         # 'sbws' failing that.
         dname = conf.getpath("paths", "log_dname")
-        os.makedirs(dname, exist_ok=True)
+        os.makedirs(dname, mode=0o700, exist_ok=True)
         fname = os.path.join(dname, "{}.log".format(args.command or "sbws"))
         # The second argument is the file mode, and it should be left alone
         mode = "a"
@@ -463,7 +458,6 @@ def _validate_destinations(conf):
                 err_tmpl.substitute(sec=sec, key=key, val=value, e=error_msg)
             )
             continue
-        assert valid
         if section.getboolean(key):
             dest_sections.append("{}.{}".format(sec, key))
     urls = {
@@ -672,7 +666,9 @@ def _validate_url(section, key):
     # when verifying certificate:
     # https://github.com/requests/requests/blob/master/requests/adapters.py#L215  # noqa
     # When the scheme is https but the protocol is not TLS, requests will hang.
-    if url.scheme != "https" and not url.netloc.startswith("127.0.0.1"):
+    # For tests we use 127.0.0.1, allow this address without https scheme.
+
+    if url.scheme != "https" and url.netloc.split(":")[0] != "127.0.0.1":
         return False, "URL scheme must be HTTPS (except for the test server)"
     return True, ""
 
@@ -683,11 +679,9 @@ def _validate_int(section, key, minimum=None, maximum=None):
     except ValueError as e:
         return False, e
     if minimum is not None:
-        assert isinstance(minimum, int)
         if value < minimum:
             return False, "Cannot be less than {}".format(minimum)
     if maximum is not None:
-        assert isinstance(maximum, int)
         if value > maximum:
             return False, "Cannot be greater than {}".format(maximum)
     return True, ""
@@ -707,11 +701,9 @@ def _validate_float(section, key, minimum=None, maximum=None):
     except ValueError as e:
         return False, e
     if minimum is not None:
-        assert isinstance(minimum, float)
         if value < minimum:
             return False, "Cannot be less than {}".format(minimum)
     if maximum is not None:
-        assert isinstance(maximum, float)
         if value > maximum:
             return False, "Cannot be greater than {}".format(maximum)
     return True, ""
